
from colibris.conf.backends import BackendMixin


ABSENT = object()


class CacheBackend(BackendMixin):
    def get(self, key):
        raise NotImplementedError

    def set(self, key, value, lifetime):
        raise NotImplementedError

    def delete(self, key):
        raise NotImplementedError




import logging

from colibris.conf import settings

from .message import EmailMessage
from .base import EmailBackend


logger = logging.getLogger(__name__)


def send(email_message):
    logger.debug('sending %s', email_message)
    return EmailBackend.get_instance().send_messages([email_message])


def send_many(email_messages):
    logger.debug('sending %d messages', len(email_messages))
    return EmailBackend.get_instance().send_messages(email_messages)


def setup():
    EmailBackend.configure(settings.EMAIL)


def is_enabled():
    return EmailBackend.is_enabled()


def is_created():
    return EmailBackend.is_created()




import logging

from colibris import monkey as __monkey  # Apply monkey patches before anything else

from colibris import authentication
from colibris import authorization
from colibris import cache
from colibris import commands
from colibris import conf
from colibris import email
from colibris import persist
from colibris import taskqueue
from colibris import template


VERSION = '0.9.2'


def setup():
    authentication.setup()
    authorization.setup()
    cache.setup()
    email.setup()
    persist.setup()
    taskqueue.setup()
    template.setup()


# Initially gather logging records into a memory handler and flush them as soon as logging is properly set up
logging.basicConfig(level=logging.DEBUG, handlers=[conf.get_logging_memory_handler()])


def is_test_mode():
    return isinstance(commands.get_command(), commands.test.TestCommand)




from aiohttp import web

from colibris import authorization
from colibris import views

from .fixtures import DUMMY_PERMISSION, ANOTHER_PERMISSION


class DummyView(views.View):
    async def get(self):
        return web.json_response({'message': 'dummy'})


class DummyViewWithClassPermission(DummyView):
    permissions = {DUMMY_PERMISSION}


class DummyViewWithMethodPermission(DummyView):
    @authorization.require_permission(DUMMY_PERMISSION)
    async def get(self):
        return await super().get()


class PermissionAuthorizationBackend(authorization.AuthorizationBackend):
    def get_actual_permissions(self, account, method, path):
        return {DUMMY_PERMISSION}


class NoPermissionAuthorizationBackend(authorization.AuthorizationBackend):
    def get_actual_permissions(self, account, method, path):
        return set()


class WrongPermissionAuthorizationBackend(authorization.AuthorizationBackend):
    def get_actual_permissions(self, account, method, path):
        return {ANOTHER_PERMISSION}


async def test_default_view_permissions(http_client_maker):
    client = await http_client_maker(routes=[('/dummy', DummyView)])
    response = await client.get('/dummy')

    assert response.status == 200


async def test_class_required_permissions_fulfilled(http_client_maker):
    client = await http_client_maker(routes=[('/dummy', DummyViewWithClassPermission)],
                                     authorization_backend=PermissionAuthorizationBackend)
    response = await client.get('/dummy')

    assert response.status == 200


async def test_class_required_permissions_no_permissions(http_client_maker):
    client = await http_client_maker(routes=[('/dummy', DummyViewWithClassPermission)],
                                     authorization_backend=NoPermissionAuthorizationBackend)
    response = await client.get('/dummy')

    assert response.status == 403


async def test_class_required_permissions_wrong_permissions(http_client_maker):
    client = await http_client_maker(routes=[('/dummy', DummyViewWithClassPermission)],
                                     authorization_backend=WrongPermissionAuthorizationBackend)
    response = await client.get('/dummy')

    assert response.status == 403


async def test_method_required_permissions_fulfilled(http_client_maker):
    client = await http_client_maker(routes=[('/dummy', DummyViewWithMethodPermission)],
                                     authorization_backend=PermissionAuthorizationBackend)
    response = await client.get('/dummy')

    assert response.status == 200


async def test_method_required_permissions_no_permissions(http_client_maker):
    client = await http_client_maker(routes=[('/dummy', DummyViewWithMethodPermission)],
                                     authorization_backend=NoPermissionAuthorizationBackend)
    response = await client.get('/dummy')

    assert response.status == 403


async def test_method_required_permissions_wrong_permissions(http_client_maker):
    client = await http_client_maker(routes=[('/dummy', DummyViewWithMethodPermission)],
                                     authorization_backend=WrongPermissionAuthorizationBackend)
    response = await client.get('/dummy')

    assert response.status == 403



from marshmallow.fields import *

from colibris import authentication


class CurrentUserField(Field):
    def __init__(self, *args, **kwargs):
        kwargs.update({
            'load_only': True
        })

        super().__init__(*args, **kwargs)

    def deserialize(self, value, attr=None, data=None, **kwargs):
        return authentication.get_account(self.context['request'])

    def serialize(self, attr, obj, accessor=None, **kwargs):
        raise NotImplementedError()



import operator
from functools import reduce

import peewee
from marshmallow import Schema, ValidationError, EXCLUDE
from marshmallow.schema import SchemaMeta, SchemaOpts

from colibris import api, persist
from colibris.views.filtering import fields, operators

TYPE_MAPPING = {
    peewee.AutoField: fields.String,
    peewee.IntegerField: fields.Integer,
    peewee.CharField: fields.String,
    peewee.BooleanField: fields.Boolean,
    peewee.DateTimeField: fields.DateTime,
    peewee.DateField: fields.Date,
    peewee.TextField: fields.String,
    peewee.FloatField: fields.Float,
    peewee.DecimalField: fields.Decimal,
    peewee.TimeField: fields.Time,
    peewee.BigIntegerField: fields.Integer,
    peewee.SmallIntegerField: fields.Integer,
    peewee.DoubleField: fields.Float,
    peewee.FixedCharField: fields.String
}

NAME_MAPPING = {
    operators.EQ: '',
    operators.GT: '__gt',
    operators.GE: '__ge',
    operators.LT: '__lt',
    operators.LE: '__le',
    operators.NOT: '__not',
    operators.LIKE: '__like',
    operators.ILIKE: '__ilike',
    operators.REGEXP: '__regexp',
}


class ModelFilterMeta(SchemaMeta):
    @classmethod
    def get_declared_fields(mcs, klass, cls_fields, inherited_fields, dict_cls):
        fields_from_model = []
        model = klass.opts.model

        if model is None:
            return dict_cls(inherited_fields + cls_fields)

        model_fields = {field.name: field for field in model._meta.sorted_fields}

        for model_field, operations in klass.opts.filter_fields.items():
            for operation in operations:
                model_field_class = model_fields[model_field].__class__
                field_class = TYPE_MAPPING[model_field_class]
                field = field_class(field=model_field, operation=operation)
                field_name = model_field + NAME_MAPPING[operation]

                fields_from_model.append(
                    (field_name, field)
                )

        return dict_cls(fields_from_model + inherited_fields + cls_fields)


class ModelFilterSchemaOpts(SchemaOpts):
    def __init__(self, meta, **kwargs):
        self.model = getattr(meta, 'model', None)
        self.filter_fields = getattr(meta, 'fields', {})
        meta.fields = ()
        super().__init__(meta, **kwargs)

        if self.model and not issubclass(self.model, persist.Model):
            raise ValueError("`model` must be a subclass of persist.Model")


class ModelFilter(Schema, metaclass=ModelFilterMeta):
    OPTIONS_CLASS = ModelFilterSchemaOpts

    class Meta(Schema.Meta):
        unknown = EXCLUDE
        model = None
        fields = ()

    def __init__(self, filter_args: dict, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.filter_args = filter_args

        try:
            self.validated_filter_args = self.load(self.filter_args)
        except ValidationError as err:
            raise api.SchemaError(details=err.messages)

    def filter_query(self, query):
        assert self.Meta.model is not None, 'The attribute "Meta.model" is required for {}'.format(self)

        conditions = []

        for filter_field, filter_value in self.validated_filter_args.items():
            op = self.fields[filter_field].operation
            field = self.fields[filter_field].field

            model_field = getattr(self.Meta.model, field)
            conditions.append(op(model_field, filter_value))

        if conditions:
            where = reduce(operator.and_, conditions)
            query = query.where(where)

        return query



import abc

from aiohttp import web, hdrs
from aiohttp_apispec import response_schema, request_schema, docs

from colibris.persist import Model


class ViewMeta(abc.ABCMeta):
    METHODS_WITH_OUTPUT = (hdrs.METH_GET, hdrs.METH_POST, hdrs.METH_PATCH, hdrs.METH_PUT)
    METHODS_WITH_INPUT = (hdrs.METH_POST, hdrs.METH_PATCH, hdrs.METH_PUT)

    def __init__(cls, name, bases, attrs):
        for http_method in hdrs.METH_ALL:
            handler_name = http_method.lower()

            if not hasattr(cls, handler_name):
                continue

            handler = getattr(cls, handler_name)
            setattr(cls, handler_name, docs()(handler))

            if getattr(cls, 'query_schema_class', None) is not None:
                setattr(cls, handler_name, request_schema(cls.query_schema_class, location='query')(handler))

            if getattr(cls, 'body_schema_class', None) is not None:
                if http_method in cls.METHODS_WITH_INPUT:
                    setattr(cls, handler_name, request_schema(cls.body_schema_class)(handler))

                if http_method in cls.METHODS_WITH_OUTPUT:
                    setattr(cls, handler_name, response_schema(cls.body_schema_class)(handler))

        if getattr(cls, 'model', None) is not None:
            assert issubclass(cls.model, Model) is True, 'The "model" should be a subclass of {}.'.format(Model)

        super().__init__(name, bases, attrs)


class View(web.View, metaclass=ViewMeta):
    authentication_required = None
    permissions = None



import math

from colibris.api.envelope import wrap_many
from marshmallow.validate import Range

from colibris import api
from colibris.schemas import fields, Schema, ValidationError, EXCLUDE

page_validator = Range(min=1, error='Value must be greater than 0.')


class PageNumberPagination:
    PAGE_SIZE = 25
    PAGE_SIZE_QUERY_PARAM = 'page_size'
    PAGE_QUERY_PARAM = 'page'

    _query_schema = type('PaginationQuerySchema', (Schema,), {
        PAGE_SIZE_QUERY_PARAM: fields.Integer(missing=PAGE_SIZE, validate=page_validator),
        PAGE_QUERY_PARAM: fields.Integer(missing=1, validate=page_validator)
    })(unknown=EXCLUDE)

    def __init__(self, query, request):
        self.query = query
        self.request = request

        try:
            query_params = self._query_schema.load(self.request.query)
        except ValidationError as err:
            raise api.SchemaError(details=err.messages)

        self.page = query_params[self.PAGE_QUERY_PARAM]
        self.page_size = query_params[self.PAGE_SIZE_QUERY_PARAM]

    def paginate_query(self):
        return self.query.paginate(self.page, self.page_size)

    def get_enveloped_data(self, data):
        count = self.query.count()
        pages = math.ceil(count / self.page_size)

        return wrap_many(
            objs=data,
            count=count,
            page=self.page,
            pages=pages,
            page_size=self.page_size
        )




import pytest

from aiohttp import web

from colibris import authentication
from colibris import authorization
from colibris import persist
from colibris.middleware.errors import handle_errors_json
from colibris.middleware.auth import handle_auth

from .authorization.fixtures import *


@pytest.fixture
def http_client_maker(aiohttp_client):

    def client(authentication_backend=None, authorization_backend=None,
               routes=None, middlewares=None):

        if middlewares is None:
            middlewares = [
                handle_errors_json,
                handle_auth
            ]

        app = web.Application(middlewares=middlewares)
        routes = routes or []

        for path, view in routes:
            app.router.add_route('*', path, view)

        if not authentication_backend:
            authentication_backend = 'colibris.authentication.null.NullBackend'

        if not authorization_backend:
            authorization_backend = 'colibris.authorization.null.NullBackend'

        authentication.AuthenticationBackend.configure({
            'backend': authentication_backend
        })

        if authorization_backend:
            authorization.AuthorizationBackend.configure({
                'backend': authorization_backend
            })

        return aiohttp_client(app)

    return client


@pytest.fixture
def database_maker():

    def database(models, **settings):
        # Use in-memory SQLite db by default
        settings.setdefault('backend', 'colibris.persist.backends.SQLiteBackend')
        if settings['backend'] == 'colibris.persist.backends.SQLiteBackend':
            settings.setdefault('database', ':memory:')

        persist.DatabaseBackend.configure(settings)
        db = persist.get_database()
        db.connect()
        persist.models.set_database(db)
        db.create_tables(models)

        return db

    return database




import importlib
import inspect
import logging.config
import logging.handlers
import os
import re
import sys

from dotenv import load_dotenv

from colibris import utils

from . import settings


ENV_DEFAULT = '.env.default'

logger = logging.getLogger(__name__)


class ImproperlyConfigured(Exception):
    pass


_default_logging_dict = settings.LOGGING  # Remember original logging config to tell if changed
_logging_memory_handler = None


def _is_setting_name(name):
    # Only consider public members that are all in capitals
    return re.match('^[A-Z][A-Z0-9_]*$', name)


def _setup_project_package():
    # Autodetect project package from main script path
    main_script = sys.argv[0]
    project_package_name = None
    if main_script.endswith('manage.py'):  # Using manage.py
        main_script = os.path.realpath(main_script)
        project_package_name = os.path.basename(os.path.dirname(main_script))
        sys.path.insert(0, os.path.dirname(os.path.dirname(main_script)))

    else:  # Using a setuptools console script wrapper
        with open(main_script, 'rt') as main_module_file:
            main_content = main_module_file.read()
            # Try simple wrapper
            m = re.search(r'from (\w+).manage import main', main_content)
            if m:
                project_package_name = m.group(1)

            else:
                # pkg_resources kind wrapper
                m = re.search(r"load_entry_point\('(\w+)=*", main_content)
                if m:
                    project_package_name = m.group(1)

    if project_package_name is None:
        raise ImproperlyConfigured('could not identify project package name')

    settings.PROJECT_PACKAGE_NAME = project_package_name

    project_package = importlib.import_module(project_package_name)
    settings.PROJECT_PACKAGE_DIR = os.path.dirname(project_package.__file__)


def _setup_env():
    # Try to load a default env file from the project package
    path = os.path.join(settings.PROJECT_PACKAGE_DIR, ENV_DEFAULT)
    if os.path.exists(path):
        load_dotenv(path)

    # Try to load an env file from the current directory
    load_dotenv('.env', override=True)


def _setup_project_settings():
    project_settings_path = '{}.settings'.format(settings.PROJECT_PACKAGE_NAME)
    project_settings_module = utils.import_module_or_none(project_settings_path)
    if project_settings_module is None:
        return

    for name, value in inspect.getmembers(project_settings_module):
        if not _is_setting_name(name):
            continue

        setattr(settings, name, value)


def _setup_logging():
    # Update default log level according to DEBUG flag
    if settings.LOGGING is _default_logging_dict and not settings.DEBUG:
        settings.LOGGING['root']['level'] = 'INFO'

    logging_config = dict(settings.LOGGING)
    logging_config['disable_existing_loggers'] = False
    utils.dict_update_rec(logging_config, settings.LOGGING_OVERRIDES)

    logging.config.dictConfig(logging_config)

    # Handle logs emitted before logging setup
    if _logging_memory_handler:
        _logging_memory_handler.setTarget(logging.getLogger())
        _logging_memory_handler.flush()


def get_logging_memory_handler():
    global _logging_memory_handler

    if _logging_memory_handler is None:
        _logging_memory_handler = logging.handlers.MemoryHandler(capacity=1e6, flushLevel=logging.CRITICAL)

    return _logging_memory_handler


def setup():
    _setup_project_package()
    _setup_env()
    _setup_project_settings()
    _setup_logging()




from aiohttp import web

from colibris import app
from colibris.conf import settings

from .base import BaseCommand


class RunServerCommand(BaseCommand):
    def execute(self, options):
        web.run_app(app.get_web_app(), host=settings.LISTEN, port=settings.PORT)




import inspect

from aiohttp import web

from colibris import api
from colibris import authentication
from colibris import authorization
from colibris import views


@web.middleware
async def handle_auth(request, handler):
    # Bail out with exception if we haven't gotten proper match_info
    if request.match_info.http_exception is not None:
        raise request.match_info.http_exception

    request = authentication.process_request(request)

    method = request.method
    path = request.match_info.route.resource.canonical

    original_handler = request.match_info.handler

    # First look for auth info in the view handler itself (class or function)
    authentication_required = authentication.get_authentication_required(original_handler)
    permissions = authorization.get_required_permissions(original_handler)

    # Then, if we've got a class-based view, look for required permissions in method function
    if inspect.isclass(original_handler) and issubclass(original_handler, views.View):
        method_func = getattr(original_handler, method.lower(), None)
        if method_func:
            method_func_authentication_required = authentication.get_authentication_required(method_func)
            method_func_permissions = authorization.get_required_permissions(method_func)

            if method_func_permissions and permissions:
                permissions = permissions.combine(method_func_permissions)

            else:
                permissions = permissions or method_func_permissions

            if method_func_authentication_required is not None:
                authentication_required = method_func_authentication_required

    # A value of None for authentication_required indicates decision based on permissions
    if authentication_required is None:
        authentication_required = bool(permissions)

    if not authentication_required and permissions:
        raise authorization.AuthorizationException('view requires permissions but does not require authentication')

    if authentication_required:
        try:
            account = authentication.authenticate(request)

        except authentication.AuthenticationException:
            raise api.UnauthenticatedException()

        if permissions is not None:
            try:
                authorization.authorize(account, method, path, original_handler, permissions)

            except authorization.AuthorizationException:
                raise api.ForbiddenException()

    response = await handler(request)
    response = authentication.process_response(request, response)

    return response




from colibris import app
from colibris import persist


def init(web_app, loop):
    # Add your coroutines to the loop here.

    pass


def get_health():
    # Determine whether your service is currently healthy.
    # Raise app.HealthException() in case of any problem.

    if not persist.connectivity_check():
        raise app.HealthException('database connectivity check failed')

    return 'healthy'



import pytest

from __packagename__ import models


TEST_USER_PASSWORD = 'eyJzdWIiOiJ0ZXN0X3VzZXIifQ'


@pytest.fixture
def test_user():
    yield models.User.create(username='test_user', key='eyJzdWIiOiJ0ZXN0X3VzZXIifQ',
                             first_name='Test', last_name='User',
                             email='testuser@example.com')



import json
import operator
import re
from functools import reduce

from marshmallow import Schema, ValidationError

from colibris import api
from colibris.views.api import APIView
from colibris.views.filtering.base import ModelFilter


class ModelView(APIView):
    model = None
    pagination_class = None
    filter_class = None
    url_identifier = 'id'
    lookup_field = 'id'

    async def get_object(self):
        identifier_value = self.request.match_info[self.url_identifier]
        query = self.get_query()
        model = query.model

        try:
            instance = query.where(getattr(model, self.lookup_field) == identifier_value).get()
        except query.model.DoesNotExist:
            raise api.ModelNotFoundException(model)

        return instance

    def get_query(self):
        model = self.get_model()

        query = model.select().order_by(self.model.id.desc())
        if self.filter_class is not None:
            filter_schema: ModelFilter = self.filter_class(self.request.query)
            query = filter_schema.filter_query(query)

        return query

    def get_model(self):
        assert self.model is not None, 'The attribute "model" is required for {}'.format(self)

        return self.model




import pytest

from colibris.authorization import permissions

from .fixtures import DUMMY_PERMISSION, ANOTHER_PERMISSION, YET_ANOTHER_PERMISSION


def test_conjunction():
    p = permissions.Permissions(and_set={DUMMY_PERMISSION, ANOTHER_PERMISSION})

    with pytest.raises(permissions.PermissionNotMet):
        p.verify(set())

    with pytest.raises(permissions.PermissionNotMet):
        p.verify({DUMMY_PERMISSION})

    with pytest.raises(permissions.PermissionNotMet):
        p.verify({ANOTHER_PERMISSION})

    p.verify({DUMMY_PERMISSION, ANOTHER_PERMISSION})


def test_disjunction():
    p = permissions.Permissions(or_set={DUMMY_PERMISSION, ANOTHER_PERMISSION})

    with pytest.raises(permissions.PermissionNotMet):
        p.verify(set())

    p.verify({DUMMY_PERMISSION})
    p.verify({ANOTHER_PERMISSION})
    p.verify({DUMMY_PERMISSION, ANOTHER_PERMISSION})


def test_conjunction_disjunction():
    p = permissions.Permissions(and_set={DUMMY_PERMISSION, ANOTHER_PERMISSION},
                                or_set={DUMMY_PERMISSION, YET_ANOTHER_PERMISSION})

    with pytest.raises(permissions.PermissionNotMet):
        p.verify(set())

    with pytest.raises(permissions.PermissionNotMet):
        p.verify({DUMMY_PERMISSION})

    with pytest.raises(permissions.PermissionNotMet):
        p.verify({ANOTHER_PERMISSION})

    with pytest.raises(permissions.PermissionNotMet):
        p.verify({YET_ANOTHER_PERMISSION})

    with pytest.raises(permissions.PermissionNotMet):
        p.verify({DUMMY_PERMISSION, YET_ANOTHER_PERMISSION})

    with pytest.raises(permissions.PermissionNotMet):
        p.verify({ANOTHER_PERMISSION, YET_ANOTHER_PERMISSION})

    p.verify({DUMMY_PERMISSION, ANOTHER_PERMISSION})
    p.verify({DUMMY_PERMISSION, ANOTHER_PERMISSION, YET_ANOTHER_PERMISSION})


def test_combine():
    p1 = permissions.Permissions(and_set={DUMMY_PERMISSION}, or_set={ANOTHER_PERMISSION})
    p2 = permissions.Permissions(and_set={ANOTHER_PERMISSION}, or_set={YET_ANOTHER_PERMISSION})

    c = p1.combine(p2)
    assert c.and_set == {DUMMY_PERMISSION, ANOTHER_PERMISSION}
    assert c.or_set == {ANOTHER_PERMISSION, YET_ANOTHER_PERMISSION}




from colibris.conf.backends import BackendMixin

from .exceptions import NoSuchAccount


_ACCOUNT_ACTION_LOGIN = 'login'
_ACCOUNT_ACTION_LOGIN_PERSISTENT = 'login_persistent'
_ACCOUNT_ACTION_LOGOUT = 'logout'

REQUEST_ACCOUNT_ACTION_KEY = 'account_action'


class AuthenticationBackend(BackendMixin):
    def extract_auth_data(self, request):
        raise NotImplementedError

    def lookup_account(self, identity):
        raise NotImplementedError

    def verify_identity(self, request, account, auth_data):
        # To be overridden if identity verification is needed.
        pass

    def authenticate(self, request):
        auth_data = self.extract_auth_data(request)
        account = self.lookup_account(auth_data)
        if account is None:
            raise NoSuchAccount()

        self.verify_identity(request, account, auth_data)

        return account

    def login(self, request, persistent):
        if persistent:
            request[REQUEST_ACCOUNT_ACTION_KEY] = _ACCOUNT_ACTION_LOGIN_PERSISTENT

        else:
            request[REQUEST_ACCOUNT_ACTION_KEY] = _ACCOUNT_ACTION_LOGIN

    def logout(self, request):
        request[REQUEST_ACCOUNT_ACTION_KEY] = _ACCOUNT_ACTION_LOGOUT

    def prepare_login_response(self, response, account, persistent):
        return response

    def prepare_logout_response(self, response):
        return response

    def process_request(self, request):
        return request

    def process_response(self, request, response):
        from . import get_account

        # Handle logins and logouts
        account_action = request.get(REQUEST_ACCOUNT_ACTION_KEY)
        if account_action:
            account = get_account(request)
            if account_action == _ACCOUNT_ACTION_LOGIN:
                response = self.prepare_login_response(response, account, persistent=False)

            elif account_action == _ACCOUNT_ACTION_LOGIN_PERSISTENT:
                response = self.prepare_login_response(response, account, persistent=True)

            elif account_action == _ACCOUNT_ACTION_LOGOUT:
                response = self.prepare_logout_response(response)

        return response




import argparse
import atexit
import os
import re
import shutil
import subprocess
import sys
import tempfile


PACKAGE_NAME_PLACEHOLDER = '__packagename__'
PROJECT_NAME_PLACEHOLDER = '__projectname__'
IGNORE_PATTERNS = ['.git', '__pycache__']

tmp_dir = None


def cleanup():
    if tmp_dir:
        tmp_dir.cleanup()


def get_skeleton_dir(skeleton):
    global tmp_dir

    if skeleton is None:  # Use default template skeleton
        return os.path.join(os.path.dirname(__file__), 'skeleton')

    elif skeleton.startswith('git@') or skeleton.startswith('http:') or skeleton.startswith('https:'):
        tmp_dir = tempfile.TemporaryDirectory()
        cmd = ['git', 'clone', skeleton, tmp_dir.name]

        subprocess.check_call(cmd)

        return tmp_dir.name

    else:  # Assuming local directory
        return skeleton


def start_project():
    atexit.register(cleanup)

    parser = argparse.ArgumentParser()
    parser.add_argument('name', help='The project name (e.g. my-project)', type=str)
    parser.add_argument('--skeleton', help='An optional template skeleton (can be a local dir or a git repo)',
                        type=str, required=False)

    args = parser.parse_args()

    project_name = args.name
    package_name = re.sub('[^a-z0-9_]', '', project_name).lower()

    skeleton_dir = get_skeleton_dir(args.skeleton)

    shutil.copytree(skeleton_dir, project_name, ignore=shutil.ignore_patterns(*IGNORE_PATTERNS))

    old_package_name = '{}/{}'.format(project_name, PACKAGE_NAME_PLACEHOLDER)
    new_package_name = '{}/{}'.format(project_name, package_name)

    shutil.move(old_package_name, new_package_name)

    if sys.platform == 'darwin':  # macOS uses BSD sed which expects extra argument to '-i'
        rename_command = 'find {} -type f | xargs sed -i "" "s/{}/{}/g"'

    else:
        rename_command = 'find {} -type f | xargs sed -i "s/{}/{}/g"'

    os.system(rename_command.format(project_name, PACKAGE_NAME_PLACEHOLDER, package_name))
    os.system(rename_command.format(project_name, PROJECT_NAME_PLACEHOLDER, project_name))

    print('project {} is ready'.format(project_name))



from colibris.docs.openapi.views import apispec_view, ui_view
from colibris.docs.openapi import STATIC_PATH, APISPEC_URL, STATIC_URL, UI_URL


def setup_openapi_ui(app):
    app.router.add_route('GET', APISPEC_URL, apispec_view)
    app.router.add_route('GET', UI_URL, ui_view)

    app.router.add_static(STATIC_URL, STATIC_PATH)




from .model import ModelBackend


# The model is assumed to have the following fields:
#  * account
#  * resource
#  * operation
#
# A permission is defined as the right to perform an operation on a resource.
# The format used is "{resource}:{operation}", e.g. "users:update".

class RightsBackend(ModelBackend):
    def __init__(self, resource_field, operation_field, **kwargs):
        self.resource_field = resource_field
        self.operation_field = operation_field

        super().__init__(**kwargs)

    def get_resource(self, right):
        return getattr(right, self.resource_field)

    def get_operation(self, right):
        return getattr(right, self.operation_field)

    def get_actual_permissions(self, account, method, path):
        # Gather all rights from all entries for the given account
        rights = self.model.select().where(self.get_account_field() == account)

        return {'{}:{}'.format(self.get_resource(r), self.get_operation(r)) for r in rights}





class AuthorizationException(Exception):
    pass


class PermissionNotMet(AuthorizationException):
    def __init__(self, permission):
        super().__init__('permission not met: {}'.format(permission))

        self.permission = permission




from colibris.conf.backends import BackendMixin


class EmailBackend(BackendMixin):
    def __init__(self, **kwargs):
        pass

    def send_messages(self, email_messages):
        raise NotImplementedError




from colibris.conf.backends import BackendMixin


class TemplateBackend(BackendMixin):
    def __init__(self, paths):
        self.paths = paths

    def render(self, template_name, context):
        raise NotImplementedError

    def render_string(self, template_str, context):
        raise NotImplementedError



from aiohttp import web
from aiohttp_apispec import docs
from colibris.views import ModelView

from colibris import app
from colibris import views
from colibris.conf import settings
from colibris.authentication import get_account
from colibris.views.generic import RetrieveUpdateDestroyModelView, ListCreateModelView

from __packagename__ import constants
from __packagename__ import models
from __packagename__ import schemas


# Here are some examples of views. Just remove what you don't need.
from colibris.views.mixins import UpdateMixin, RetrieveMixin


class HomeView(views.View):
    # Show the API docs when visiting the index of the service
    async def get(self):
        raise web.HTTPFound(settings.API_DOCS_URL)


class HealthView(views.View):
    @docs(tags=['Service'],
          summary='The health check endpoint')
    async def get(self):
        h = await app.get_health()
        return web.json_response(h)


class MeView(ModelView, RetrieveMixin, UpdateMixin):
    authentication_required = True
    body_schema_class = schemas.UserSchema
    model = models.User

    async def get_object(self):
        user = get_account(self.request)
        return user


class UsersView(ListCreateModelView):
    permissions = {constants.ROLE_ADMIN}
    body_schema_class = schemas.UserSchema
    model = models.User

    @docs(tags=['Users'], summary='List all users')
    async def get(self):
        return await super().get()

    @docs(tags=['Users'], summary='Add a new user')
    async def post(self):
        return await super().post()


class UserView(RetrieveUpdateDestroyModelView):
    permissions = {constants.ROLE_ADMIN}
    body_schema_class = schemas.UserSchema
    model = models.User

    @docs(tags=['Users'], summary='Reveal details about a specific user')
    async def get(self):
        return await super().get()

    @docs(tags=['Users'], summary='Update an existing user')
    async def patch(self):
        return await super().patch()

    @docs(tags=['Users'], summary='Update an existing user')
    async def put(self):
        return await super().put()

    @docs(tags=['Users'], summary='Delete a user')
    async def delete(self):
        return await super().delete()




import os

from peewee import PostgresqlDatabase, MySQLDatabase, DatabaseError, mysql
from playhouse.sqlite_ext import SqliteExtDatabase

import colibris

from colibris.conf.backends import BackendMixin


class DatabaseBackend(BackendMixin):
    _dropped = False

    def create(self):
        if not colibris.is_test_mode():
            raise DatabaseError('refusing to create database in non-testing mode')

        self._create(self.database)
        self._dropped = False

    def drop(self):
        if not colibris.is_test_mode():
            raise DatabaseError('refusing to drop database in non-testing mode')

        if not self.is_closed():
            self.close()

        self._drop(self.database)
        self._dropped = True

    def is_dropped(self):
        return self._dropped

    def _create(self, name):
        raise NotImplementedError

    def _drop(self, name):
        raise NotImplementedError


class PostgreSQLBackend(PostgresqlDatabase, DatabaseBackend):
    def _one_shot_sql(self, sql):
        import psycopg2
        from psycopg2.extensions import ISOLATION_LEVEL_AUTOCOMMIT

        conn = psycopg2.connect(dbname='postgres',
                                host=self.connect_params['host'],
                                port=self.connect_params.get('port'),
                                user=self.connect_params['user'],
                                password=self.connect_params['password'])

        conn.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)

        cur = conn.cursor()
        cur.execute(sql)

        conn.close()

    def _create(self, name):
        self._one_shot_sql('CREATE DATABASE "{name}"'.format(name=name))

    def _drop(self, name):
        self._one_shot_sql('DROP DATABASE "{name}"'.format(name=name))


class MySQLBackend(MySQLDatabase, DatabaseBackend):
    def _one_shot_sql(self, sql):
        conn = mysql.connect(host=self.connect_params['host'],
                             port=self.connect_params.get('port'),
                             user=self.connect_params['user'],
                             passwd=self.connect_params['password'])
        cur = conn.cursor()
        cur.execute(sql)

        conn.close()

    def _create(self, name):
        self._one_shot_sql('CREATE DATABASE {name}'.format(name=name))

    def _drop(self, name):
        self._one_shot_sql('DROP DATABASE {name}'.format(name=name))


class SQLiteBackend(SqliteExtDatabase, DatabaseBackend):
    def _create(self, name):
        pass  # SQLite automatically creates the DB file when opened

    def _drop(self, name):
        os.remove(name)




import os
import re

try:
    # Speeds up load time by eliminating the pkg_resources overhead.
    import fastentrypoints

except ImportError:
    pass

from setuptools import setup, find_packages


PROJECT_PACKAGE_NAME = '__packagename__'
PROJECT_NAME = '__projectname__'


def package_data_rec(package, directory):
    paths = []
    for path, directories, filenames in os.walk(os.path.join(package, directory)):
        for filename in filenames:
            paths.append(os.path.join(path, filename)[len(package) + 1:])

    return paths


def find_version():
    with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), PROJECT_PACKAGE_NAME, '__init__.py')) as f:
        m = re.search(r"VERSION\s*=\s*'(.*?)'", f.read())
        if m:
            return m.group(1)

    return 'unknown'


setup(
    name=PROJECT_PACKAGE_NAME,
    version=find_version(),
    install_requires=[
        'colibris'
    ],
    url='',
    license='',
    description='Project description',
    packages=find_packages(include=PROJECT_PACKAGE_NAME + '/*'),
    package_data={
        PROJECT_PACKAGE_NAME: (package_data_rec(PROJECT_PACKAGE_NAME, 'templates') +
                               package_data_rec(PROJECT_PACKAGE_NAME, 'static') +
                               ['.env.default'])
    },
    entry_points={
        'console_scripts': [
            '{project_name}={package_name}.manage:main'.format(project_name=PROJECT_NAME,
                                                               package_name=PROJECT_PACKAGE_NAME),
        ]
    }
)




import logging

from colibris import taskqueue

from .base import BaseCommand


logger = logging.getLogger(__name__)


class RunWorkerCommand(BaseCommand):
    def add_arguments(self, parser):
        taskqueue.add_worker_arguments(parser)

    def execute(self, options):
        taskqueue.run_worker(options)



from colibris.conf.schemas import fields, SettingsSchema

from . import constants


class GeneralSettingsSchema(SettingsSchema):
    DEBUG = fields.Boolean()
    LISTEN = fields.String()
    PORT = fields.Integer()
    SECRET_KEY = fields.String()

    class Meta:
        sensible_fields = ['SECRET_KEY']


DEBUG = True

LISTEN = '0.0.0.0'
PORT = 8888

SECRET_KEY = 'replace-me-with-random-ascii-string-or-supply-via-environment'

DATABASE = {
    'backend': 'colibris.persist.SQLiteBackend',
    'name': '__packagename__.db'
}

AUTHENTICATION = {
    'backend': 'colibris.authentication.apikey.APIKeyBackend',
    'model': '__packagename__.models.User',
    'key_field': 'key',
}

AUTHORIZATION = {
    'backend': 'colibris.authorization.role.RoleBackend',
    'role_field': 'role',
    'order': constants.ROLES
}

GeneralSettingsSchema().load_from_env(target_settings=globals())



from os.path import abspath, join, dirname

from colibris.conf import settings

STATIC_PATH = abspath(join(dirname(__file__), 'swagger'))

UI_URL = settings.API_DOCS_URL
STATIC_URL = '{}/static'.format(UI_URL)
APISPEC_URL = '{}/apispec'.format(UI_URL)



import operator

from peewee import OP, Expression

EQ = operator.eq
GT = operator.gt
GE = operator.ge
LT = operator.lt
LE = operator.le
NOT = operator.ne
LIKE = lambda l, r: Expression(l, OP.LIKE, r)
ILIKE = lambda l, r: Expression(l, OP.ILIKE, r)
REGEXP = lambda l, r: Expression(l, OP.REGEXP, r)
IREGEXP = lambda l, r: Expression(l, OP.IREGEXP, r)



from colibris.views.mixins import ListMixin, CreateMixin, RetrieveMixin, UpdateMixin, DestroyMixin
from colibris.views.model import ModelView


class ListCreateModelView(ModelView, ListMixin, CreateMixin):
    pass


class RetrieveUpdateDestroyModelView(ModelView, RetrieveMixin, UpdateMixin, DestroyMixin):
    pass



import pytest

from aiohttp import web
from marshmallow import Schema, fields

from colibris.views import APIView
from colibris.middleware.errors import handle_errors_json


class ItemSchema(Schema):
    name = fields.String()
    info = fields.String()
    count = fields.Integer()


class QuerySchema(Schema):
    count = fields.Integer()


class ItemsView(APIView):
    body_schema_class = ItemSchema
    query_schema_class = QuerySchema

    async def get(self):
        query = self.get_validated_query()

        return web.json_response({'query': query})

    async def post(self):
        query = self.get_validated_query()
        body = self.get_validated_body()

        return web.json_response({'query': query, 'body': body})


@pytest.fixture
async def http_client(http_client_maker):
    return await http_client_maker(middlewares=[handle_errors_json],
                                   routes=[('/items', ItemsView)])


async def test_get(http_client):
    sent_args = {
        'count': '222'
    }
    expected_args = {
        'count': 222
    }

    response = await http_client.get('/items', params=sent_args)

    assert response.status == 200

    data = await response.json()
    query = data['query']

    assert expected_args == query


async def test_post_body(http_client):
    sent_data = {
        'name': 'Risus Fusce',
        'info': 'Egestas Lorem Sit Fringilla',
        'count': '22',
    }
    expected_data = {
        'name': 'Risus Fusce',
        'info': 'Egestas Lorem Sit Fringilla',
        'count': 22,
    }

    response = await http_client.post('/items', json=sent_data)

    assert response.status == 200

    data = await response.json()
    body = data['body']

    assert expected_data == body


async def test_post_query(http_client):
    sent_data = {
        'count': '22',
    }
    expected_data = {
        'count': 22,
    }

    response = await http_client.post('/items', params=sent_data)

    assert response.status == 200

    data = await response.json()
    query = data['query']

    assert expected_data == query



import pytest

from colibris.middleware.errors import handle_errors_json
from colibris.schemas import ModelSchema

from colibris import persist
from colibris.views.filtering import fields, operators
from colibris.views.filtering.base import ModelFilter
from colibris.views.generic import ListCreateModelView


class Item(persist.Model):
    name = persist.CharField()
    count = persist.IntegerField()
    info = persist.TextField()


class ItemSchema(ModelSchema):
    class Meta:
        model = Item


class ExplicitFieldsFilter(ModelFilter):
    name = fields.String(field='name', operation=operators.EQ)
    name__regexp = fields.String(field='name', operation=operators.REGEXP)
    name__not = fields.String(field='name', operation=operators.NOT)
    name__ilike = fields.String(field='name', operation=operators.ILIKE)
    count__gt = fields.String(field='count', operation=operators.GT)
    count__ge = fields.String(field='count', operation=operators.GE)
    count__lt = fields.String(field='count', operation=operators.LT)
    count__le = fields.String(field='count', operation=operators.LE)

    class Meta:
        model = Item


class GeneratedFieldsFilter(ModelFilter):
    class Meta:
        model = Item
        fields = {
            'name': (operators.EQ, operators.REGEXP, operators.NOT, operators.ILIKE),
            'count': (operators.GT, operators.GE, operators.LT, operators.LE)
        }


class ExplicitFieldsFilterItemsView(ListCreateModelView):
    body_schema_class = ItemSchema
    model = Item
    filter_class = ExplicitFieldsFilter


class GeneratedFieldsFilterItemsView(ListCreateModelView):
    body_schema_class = ItemSchema
    model = Item
    filter_class = GeneratedFieldsFilter


MODELS = [Item]


def setup_db(models):
    settings = {
        'backend': 'colibris.persist.backends.SQLiteBackend',
        'database': ':memory:',
        'regexp_function': True
    }

    persist.DatabaseBackend.configure(settings)
    db = persist.get_database()
    db.connect()
    persist.models.set_database(db)
    db.create_tables(models)


def setup_module(module):
    setup_db(MODELS)

    Item.create(name='Justo', count=1, info='Ridiculus Risus')
    Item.create(name='Justo', count=2, info='Ligula Dolor')
    Item.create(name='Vestibulum', count=3, info='Justo Dolor')
    Item.create(name='Sit', count=4, info='Tortor Sollicitudin')
    Item.create(name='Porta', count=5, info='Etiam Consectetur')
    Item.create(name='Mattis', count=6, info='Parturient Vulputate')


@pytest.fixture
async def http_client(http_client_maker):
    return await http_client_maker(middlewares=[handle_errors_json],
                                   routes=[
                                       ('/some-items', ExplicitFieldsFilterItemsView),
                                       ('/same-items', GeneratedFieldsFilterItemsView), ]
                                   )


class TestFilterExplicitFields:
    @classmethod
    def setup_class(cls):
        cls.url = '/some-items'

    async def test_single_filter_eq(self, http_client):
        response = await http_client.get(self.url, params={'name': 'Justo'})
        assert response.status == 200

        data = await response.json()
        assert isinstance(data, list)
        assert len(data) == 2
        assert data[0]['name'] == 'Justo'
        assert data[1]['name'] == 'Justo'

    async def test_single_filter_regexp(self, http_client):
        response = await http_client.get(self.url, params={'name__regexp': '.*J.*u.*s.*t.*o.*'})
        assert response.status == 200

        data = await response.json()
        assert isinstance(data, list)
        assert len(data) == 2
        assert data[0]['name'] == 'Justo'
        assert data[1]['name'] == 'Justo'

    async def test_single_filter_not(self, http_client):
        response = await http_client.get(self.url, params={'name__not': 'Justo'})
        assert response.status == 200

        data = await response.json()
        assert isinstance(data, list)
        assert len(data) == 4

    async def test_single_filter_like(self, http_client):
        response = await http_client.get(self.url, params={'name__ilike': '%%sto%'})
        assert response.status == 200

        data = await response.json()
        assert isinstance(data, list)
        assert len(data) == 2
        assert data[0]['name'] == 'Justo'
        assert data[1]['name'] == 'Justo'

    async def test_single_filter_gt(self, http_client):
        response = await http_client.get(self.url, params={'count__gt': 3})
        assert response.status == 200

        data = await response.json()
        assert isinstance(data, list)
        assert len(data) == 3

    async def test_single_filter_ge(self, http_client):
        response = await http_client.get(self.url, params={'count__ge': 3})
        assert response.status == 200

        data = await response.json()
        assert isinstance(data, list)
        assert len(data) == 4

    async def test_single_filter_lt(self, http_client):
        response = await http_client.get(self.url, params={'count__lt': 6})
        assert response.status == 200

        data = await response.json()
        assert isinstance(data, list)
        assert len(data) == 5

    async def test_single_filter_le(self, http_client):
        response = await http_client.get(self.url, params={'count__le': 6})
        assert response.status == 200

        data = await response.json()
        assert isinstance(data, list)
        assert len(data) == 6


class TestFilterGeneratedFields(TestFilterExplicitFields):
    @classmethod
    def setup_class(cls):
        cls.url = '/same-items'




import os
import re

from setuptools import setup, find_packages


def package_data_rec(package, directory):
    paths = []
    for path, directories, filenames in os.walk(os.path.join(package, directory)):
        for filename in filenames:
            paths.append(os.path.join(path, filename)[len(package) + 1:])

    return paths


def find_version():
    with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'colibris', '__init__.py')) as f:
        m = re.search(r"VERSION\s*=\s*'(.*?)'", f.read())
        if m:
            return m.group(1)

    return 'unknown'


setup(
    name='colibris',
    version=find_version(),
    install_requires=[
        'aiohttp>=3.5.0,<3.7',
        'aiohttp-apispec>=1.5',
        'apispec>=3.0.0',
        'async-timeout',
        'marshmallow>=3.0.0b19,<3.8',
        'marshmallow_peewee>=2.3.0,<3.1',
        'peewee>=3.9',
        'peewee-migrate>=1.1.6',
        'python-dotenv',
        'webargs>=5.2.0,<6.0'
    ],
    url='https://github.com/colibris-framework/colibris',
    license='BSD',
    description='A lightweight framework for RESTful microservices',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    author='The Colibris Team',
    packages=find_packages(include='colibris/*'),
    package_data={
        'colibris': (package_data_rec('colibris', 'skeleton') +
                     package_data_rec('colibris', 'docs'))
    },
    entry_points={
        'console_scripts': [
            'colibris-start-project=colibris.startproject:start_project',
        ]
    }
)




import argparse
import sys

import colibris


class BaseCommand:
    PROG = None
    USAGE = None
    DESCRIPTION = None
    EPILOG = None
    ADD_HELP = True

    def __init__(self, args):
        self.args = args
        self.parser = self.make_argument_parser()
        self.add_arguments(self.parser)
        self.initialize()

    def initialize(self):
        colibris.setup()

    def run(self):
        options = self.parse_arguments(self.parser, self.args)
        return self.execute(options)

    def make_argument_parser(self):
        return argparse.ArgumentParser(prog=self.PROG or self.make_prog(),
                                       usage=self.USAGE,
                                       description=self.DESCRIPTION,
                                       epilog=self.EPILOG,
                                       add_help=self.ADD_HELP)

    def make_prog(self):
        return '{arg0} {cmd}'.format(arg0=sys.argv[0], cmd=self.get_name())

    def add_arguments(self, parser):
        pass

    def parse_arguments(self, parser, args):
        return parser.parse_args(args)

    def execute(self, options):
        raise NotImplementedError

    @classmethod
    def get_name(cls):
        return cls.__name__[0:-7].lower()




from colibris.conf.backends import BackendMixin


class AuthorizationBackend(BackendMixin):
    def get_actual_permissions(self, account, method, path):
        raise NotImplementedError

    def authorize(self, account, method, path, handler, required_permissions):
        required_permissions.verify(self.get_actual_permissions(account, method, path))




import types


DUMMY_PERMISSION = 'dummy_permission'
ANOTHER_PERMISSION = 'another_permission'
YET_ANOTHER_PERMISSION = 'yet_another_permission'

DUMMY_ACCOUNT = types.SimpleNamespace(role=DUMMY_PERMISSION)
ANOTHER_ACCOUNT = types.SimpleNamespace(role=ANOTHER_PERMISSION)



import json
from json import JSONDecodeError

from marshmallow import ValidationError

from colibris import api
from colibris.views.base import View

REQUEST_BODY_KEY = 'body_text'


class APIView(View):
    body_schema_class = None
    query_schema_class = None

    async def _iter(self):
        await self.prepare()

        return await super()._iter()

    async def prepare(self):
        self.request[REQUEST_BODY_KEY] = await self.request.text()

    def get_body_schema(self, *args, **kwargs):
        assert self.body_schema_class is not None, 'The attribute "body_schema_class" is required for {}'.format(self)

        kwargs.update({
            'context': {'request': self.request}
        })

        schema = self.body_schema_class(*args, **kwargs)

        return schema

    def get_query_schema(self, *args, **kwargs):
        assert self.query_schema_class is not None, 'The attribute "query_schema_class" is required for {}'.format(self)

        kwargs.update({
            'context': {'request': self.request}
        })

        schema = self.query_schema_class(*args, **kwargs)

        return schema

    def get_validated_body(self, schema=None):
        if schema is None:
            schema = self.get_body_schema()

        json_payload = self.get_request_body()

        try:
            data = schema.load(json_payload)
        except ValidationError as err:
            raise api.SchemaError(details=err.messages)

        return data

    def get_request_body(self):
        if not self.request.body_exists:
            return {}

        try:
            json_payload = json.loads(self.request[REQUEST_BODY_KEY])
        except JSONDecodeError:
            raise api.JSONParseError()

        return json_payload

    def get_validated_query(self, schema=None):
        if schema is None:
            schema = self.get_query_schema()

        query = self.get_request_query()

        try:
            data = schema.load(query)
        except ValidationError as err:
            raise api.SchemaError(details=err.messages)

        return data

    def get_request_query(self):
        return self.request.query




from .base import AuthenticationBackend


class NullBackend(AuthenticationBackend):
    _DUMMY_ACCOUNT = {}

    def __init__(self, **kwargs):
        pass

    def extract_auth_data(self, request):
        return None, None

    def lookup_account(self, identity):
        return self._DUMMY_ACCOUNT

    def verify_identity(self, request, account, auth_data):
        pass

    def authenticate(self, request):
        return self._DUMMY_ACCOUNT




from colibris.authorization.role import RoleBackend

from .fixtures import DUMMY_PERMISSION, ANOTHER_PERMISSION, YET_ANOTHER_PERMISSION, DUMMY_ACCOUNT, ANOTHER_ACCOUNT


def test_extract_role():
    backend = RoleBackend(role_field='role')
    assert backend.get_role(account=DUMMY_ACCOUNT) == DUMMY_PERMISSION


def test_actual_permissions_simple():
    backend = RoleBackend(role_field='role')
    assert backend.get_actual_permissions(account=DUMMY_ACCOUNT, method='GET', path='/') == {DUMMY_PERMISSION}


def test_actual_permissions_order():
    backend = RoleBackend(role_field='role', order=[DUMMY_PERMISSION, ANOTHER_PERMISSION, YET_ANOTHER_PERMISSION])
    permissions = backend.get_actual_permissions(account=ANOTHER_ACCOUNT, method='GET', path='/')
    assert permissions == {DUMMY_PERMISSION, ANOTHER_PERMISSION}



from colibris import persist


# Here are some examples of models. Just remove what you don't need.
from __packagename__ import constants


class User(persist.Model):
    id = persist.AutoField()
    name = persist.CharField(max_length=128, index=True, unique=True)
    key = persist.CharField(max_length=128)
    email = persist.CharField(max_length=128, null=True)
    role = persist.CharField(max_length=10, choices=[(r, r) for r in constants.ROLES])




from colibris.conf import settings

from .base import AuthorizationBackend
from .exceptions import *
from .permissions import *


def authorize(account, method, path, handler, permissions):
    return AuthorizationBackend.get_instance().authorize(account, method, path, handler, permissions)


def setup():
    AuthorizationBackend.configure(settings.AUTHORIZATION)




from .exceptions import PermissionNotMet


class Permissions:
    def __init__(self, and_set=None, or_set=None):
        self.and_set = set(and_set or ())
        self.or_set = set(or_set or ())

    def combine(self, permissions):
        return Permissions(self.and_set | permissions.and_set, self.or_set | permissions.or_set)

    def verify(self, actual_permissions):
        # Verify permissions in and set
        for p in self.and_set:
            if p not in actual_permissions:
                raise PermissionNotMet(p)

        # Verify permissions in or set
        if not self.or_set:
            return

        permissions = self.or_set & actual_permissions
        if len(permissions) == 0:
            raise PermissionNotMet(list(self.or_set)[0])

    def __str__(self):
        s = ''
        if self.and_set:
            s = ' & '.join(self.and_set)

        if self.or_set:
            or_set_str = ' | '.join(self.or_set)
            if s:
                s += ' & (' + or_set_str + ')'

            else:
                s = or_set_str

        return s or '*'

    def __bool__(self):
        return bool(self.and_set) or bool(self.or_set)


def _require_permissions(and_set=None, or_set=None):
    and_set = and_set or set()
    or_set = or_set or set()

    new_permissions = Permissions(and_set, or_set)

    def decorator(handler):
        # Combine any existing permissions with the new ones
        permissions = get_required_permissions(handler) or Permissions()
        handler.permissions = permissions.combine(new_permissions)

        return handler

    return decorator


def require_permission(permission):
    return _require_permissions(and_set=[permission])


def require_one_permission(permissions):
    return _require_permissions(or_set=permissions)


def require_all_permissions(permissions):
    return _require_permissions(and_set=permissions)


def get_required_permissions(handler):
    permissions = getattr(handler, 'permissions', None)
    if permissions is None:
        return

    # Permissions can be stored as:
    #  * a set of permissions
    #  * a Permissions instance

    # Normalize possible ways of storing permissions
    if not isinstance(permissions, Permissions):
        permissions = Permissions(and_set=permissions)

    return permissions
